<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class MapaFaturamentoFactory
{
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }


    // select principal
    public function getMapaFaturamento($parametros, $page, $limit, $conditions, $order)
    {   
      $startPos = ($page - 1) * $limit;
      $finalPos = $page * $limit;
      $sql  = <<<SQL
       SELECT  COUNT(1) OVER () TOTAL_LINES_ZHNEXT, C.* FROM (SELECT EST.*, CASE WHEN ((DIFERENCA1 = 0) OR (DIFERENCA2 = 0) OR (DIFERENCA3 = 0)) 
		 THEN 'S'
		 ELSE 'N'
	   END AS NAO_DIFERENCA,
	   CASE WHEN ((DESVIO1 > 0) OR (DESVIO2 > 0) OR (DESVIO3 > 0)) 
		 THEN 'S'
		 ELSE 'N'
	   END AS DESVIO_POSITIVO,
	   CASE WHEN ((DESVIO1 < 0) OR (DESVIO2 < 0) OR (DESVIO3 < 0)) 
		 THEN 'S'
		 ELSE 'N'
	   END AS DESVIO_NEGATIVO,
	   CASE WHEN ((DESVIO1 = 0) OR (DESVIO2 = 0) OR (DESVIO3 = 0)) 
		 THEN 'S'
		 ELSE 'N'
	   END AS DESVIO_ZERADO,
	   '' AS DESVIO_SINAL,
     :TPESTIMADO AS TPESTIMADO 
	FROM FCT_ESTIMADO_MAPAFATURA ('SERVICO', :TPESTIMADO, NULL, NULL, :NRORG) EST
	ORDER BY EST.NMSERVORCA) C
              $conditions
              $order
              OFFSET $startPos ROWS FETCH NEXT $limit ROWS ONLY
SQL;

        $result = $this->entityManager->getConnection()->fetchAll($sql, $parametros);

        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $limit, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }

    // Trás parceiros de negócio
    public function getMapaFaturamentoParc($parametros, $page, $limit, $conditions, $order)
    {
      $startPos = ($page - 1) * $limit;
      $finalPos = $page * $limit;
      $sql  = <<<SQL
       SELECT  COUNT(1) OVER () TOTAL_LINES_ZHNEXT, C.* FROM (SELECT EST.*, CASE WHEN ((DIFERENCA1 = 0) OR (DIFERENCA2 = 0) OR (DIFERENCA3 = 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS NAO_DIFERENCA,
                       CASE WHEN ((DESVIO1 > 0) OR (DESVIO2 > 0) OR (DESVIO3 > 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_POSITIVO,
                       CASE WHEN ((DESVIO1 < 0) OR (DESVIO2 < 0) OR (DESVIO3 < 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_NEGATIVO,
                       CASE WHEN ((DESVIO1 = 0) OR (DESVIO2 = 0) OR (DESVIO3 = 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_ZERADO,
                       '' AS DESVIO_SINAL,
                       :TPESTIMADO AS TPESTIMADO  
                    FROM FCT_ESTIMADO_MAPAFATURA ('PARCEIRO', :TPESTIMADO, :CDSERVORCA, NULL, :NRORG) EST
                    ORDER BY EST.NMPARCEIRO) C
              $conditions
              $order
              OFFSET $startPos ROWS FETCH NEXT $limit ROWS ONLY
SQL;

        $result = $this->entityManager->getConnection()->fetchAll($sql, $parametros);

        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $limit, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }

    // trás os contratos
    public function getMapaFaturamentoCli($parametros, $page, $limit, $conditions, $order)
    {
      $startPos = ($page - 1) * $limit;
      $finalPos = $page * $limit;
      $sql  = <<<SQL
       SELECT  COUNT(1) OVER () TOTAL_LINES_ZHNEXT, C.* FROM (SELECT EST.*, CASE WHEN ((DIFERENCA1 = 0) OR (DIFERENCA2 = 0) OR (DIFERENCA3 = 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS NAO_DIFERENCA,
                       CASE WHEN ((DESVIO1 > 0) OR (DESVIO2 > 0) OR (DESVIO3 > 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_POSITIVO,
                       CASE WHEN ((DESVIO1 < 0) OR (DESVIO2 < 0) OR (DESVIO3 < 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_NEGATIVO,
                       CASE WHEN ((DESVIO1 = 0) OR (DESVIO2 = 0) OR (DESVIO3 = 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_ZERADO,
                       '' AS DESVIO_SINAL,
                       :TPESTIMADO AS TPESTIMADO   
                    FROM FCT_ESTIMADO_MAPAFATURA ('CONTRATO', :TPESTIMADO, :CDSERVORCA, :NRPARCNEGOCIO, :NRORG) EST
                    ORDER BY EST.CONTRATO) C
              $conditions
              $order
              OFFSET $startPos ROWS FETCH NEXT $limit ROWS ONLY
SQL;

        $result = $this->entityManager->getConnection()->fetchAll($sql, $parametros);

        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $limit, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }

    // select do xml - todos os dados reunidos
    public function getDadosFaturamentoCompleto($parametros, $page, $limit, $conditions, $order)
    {
      $startPos = ($page - 1) * $limit;
      $finalPos = $page * $limit;
      $sql  = <<<SQL
       SELECT  COUNT(1) OVER () TOTAL_LINES_ZHNEXT, C.* FROM (SELECT EST.*, CASE WHEN ((DIFERENCA1 = 0) OR (DIFERENCA2 = 0) OR (DIFERENCA3 = 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS NAO_DIFERENCA,
                       CASE WHEN ((DESVIO1 > 0) OR (DESVIO2 > 0) OR (DESVIO3 > 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_POSITIVO,
                       CASE WHEN ((DESVIO1 < 0) OR (DESVIO2 < 0) OR (DESVIO3 < 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_NEGATIVO,
                       CASE WHEN ((DESVIO1 = 0) OR (DESVIO2 = 0) OR (DESVIO3 = 0)) 
                         THEN 'S'
                         ELSE 'N'
                       END AS DESVIO_ZERADO,
                       '' AS DESVIO_SINAL,
                       :TPESTIMADO AS TPESTIMADO  
                    FROM FCT_ESTIMADO_MAPAFATURA ('EXPORTACAO', :TPESTIMADO, NULL, NULL, :NRORG) EST
                    ORDER BY CDSERVORCA DESC, NRPARCNEGOCIO DESC, NRSEQCONTRATO DESC) C
              $conditions
              $order
              OFFSET $startPos ROWS FETCH NEXT $limit ROWS ONLY
SQL;

        $result = $this->entityManager->getConnection()->fetchAll($sql, $parametros);

        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $limit, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }
}
