import { Utilities } from '@/util/Utilities';
import {
    Modal, ModalService, Select, TextInput, Text, FileInput, Form, LoadingService, Number, DialogService, Button,
} from '@zeedhi/common';
import {
    Config, Http, IEventParam, Loader, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Sm00004006TabAnexosDocumentoContratoController {
    private basicModal!: Modal;

    public gridDocumentoContrato = Loader.getInstance('Sm00004006DocumentoContratoController');
    
    async exibirModalAnexoDocumento() {
        this.basicModal = await ModalService.createFromJson('sm_00004_006_documento_contrato/modal/sm_00004_006_anexo_documento_contrato', true);
        this.basicModal.show();
    }

    public fecharModalAnexoDocumento() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModalAnexoDocumento() {
        await this.exibirModalAnexoDocumento();
        Metadata.getInstance<Text>('titleText').text = 'Novo Anexo';
    }

    public async abrirModalEdicaoAnexoDocumento({ row }: any) {
        await this.exibirModalAnexoDocumento();
        Metadata.getInstance<Text>('titleText').text = 'Edição de Anexo';
        Metadata.getInstance<TextInput>('NRCONTRATODOCIDIOMA').value = row.NRCONTRATODOCIDIOMA;
        Metadata.getInstance<TextInput>('NRSEQCONTDOC').value = row.NRSEQCONTDOC;
        Metadata.getInstance<Select>('CDIDIOM').value = row.CDIDIOM;
        Metadata.getInstance<TextInput>('IDPADRAO_OLD').value = row.IDPADRAO;
        Metadata.getInstance<Select>('IDATIVO').value = row.IDATIVO;
        Metadata.getInstance<TextInput>('NMARQUIVO').value = row.NMARQUIVO;
        Metadata.getInstance<TextInput>('DSCAMINHOARQ').value = row.DSCAMINHOARQ;
        Metadata.getInstance<TextInput>('DSTIPO').value = row.DSTIPO;
        Metadata.getInstance<Number>('VRTAMANHO').value = row.VRTAMANHO;

        if (Metadata.getInstance<TextInput>('NMARQUIVO').value != null) {
            Metadata.getInstance<FileInput>('ARQUIVO').isVisible = false;
        } else {
            Metadata.getInstance<FileInput>('ARQUIVO').isVisible = true;
        }
    }

    public async inserirAnexoDocumentoContrato() {
        const form = Metadata.getInstance<Form>('form_anexo_documento_contrato');
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-contrato');
        if (Utilities.validateForm(form)) {
            const file = await Utilities.buildArrayToUpload('ARQUIVO');
            LoadingService.show();
            const params = {
                NRCONTRATODOCIDIOMA: Metadata.getInstance<TextInput>('NRCONTRATODOCIDIOMA').value,
                NRSEQCONTDOC: this.gridDocumentoContrato.currentRow.NRSEQCONTDOC,
                CDIDIOM: Metadata.getInstance<Select>('CDIDIOM').value,
                IDPADRAO: Metadata.getInstance<Select>('CDIDIOM').selectValue.IDPADRAO,
                IDPADRAO_OLD: Metadata.getInstance<TextInput>('IDPADRAO_OLD').value,
                IDATIVO: this.gridDocumentoContrato.currentRow.IDATIVO,
                NMARQUIVO: Metadata.getInstance<TextInput>('NMARQUIVO').value,
                DSCAMINHOARQ: Metadata.getInstance<TextInput>('DSCAMINHOARQ').value,
                DSTIPO: Metadata.getInstance<TextInput>('DSTIPO').value,
                VRTAMANHO: Metadata.getInstance<Number>('VRTAMANHO').value,
                ARQUIVO: file,
            };
            Http.post('/inserirAnexoDocumentoContrato', params).then(() => {
                this.fecharModalAnexoDocumento();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
                if (error.response.data.error === 'EXISTE') {
                    this.showErrorDialogValidacaoAnexo();
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showErrorDialogValidacaoAnexo() {
        DialogService.show({
            name: 'error-dialog-validacao',
            type: 'error',
            title: 'Atenção',
            text: 'Não é possível adicionar outro registro do mesmo idioma para o mesmo documento de contrato',
        });
    }

    public cliqueCelula({ column }: any) {
        let preventRowClick = false;
        if (column.name === 'DELETARANEXO' || column.name === 'DOWNLOADANEXO' || column.name === 'ATUALIZARANEXO') {
            preventRowClick = true;
        }
        return preventRowClick;
    }

    public habilitarBotaoGrid({ row }: any) {
        return (row.NMARQUIVO != null);
    }

    public habilitarBotaoAtualizarGrid({ row }: any) {
        return (row.NMARQUIVO != null && row.NRCONTRATODOCIDIOMA != null);
    }
    
    public downloadAnexoModeloDocumentoContrato({ row }: any) {
        const nrcontratodocidioma = row.NRCONTRATODOCIDIOMA;
        const nrseqcontdoc = row.NRSEQCONTDOC;
        const idpadrao = row.IDPADRAO;
        window.open(`${Config.endPoint}downloadAnexoModeloDocumentoContrato?idpadrao=${idpadrao}&nrcontratodocidioma=${nrcontratodocidioma}&nrseqcontdoc=${nrseqcontdoc}`);
    }

    public dialogDeletarAnexoModeloDocumentoContrato({ row }: IEventParam<Button>) {
        DialogService.show({
            name: 'dialog-deletar-anexo',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente excluir o documento anexado?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarAnexoModeloDocumentoContrato(row);
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletarAnexoModeloDocumentoContrato(row: any) {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-contrato');
        LoadingService.show();
        Http.post('/deletarAnexoDocumentoContrato', {
            row,
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public dialogDeletarAnexo({ event } : IEventParam<TekGrid>) {
        event?.preventDefault();
        DialogService.show({
            name: 'dialog-deletar-anexo-documento-contrato',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente excluir todos os registros selecionados? Seu documento anexado será perdido.',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarAnexoDocumentoContrato();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletarAnexoDocumentoContrato() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-contrato');
        LoadingService.show();
        Http.post('/deletarAnexoDocumentoContrato', grid.selectedRows).then(() => {
            grid.selectedRows = [];
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public dialogAtualizarAnexoModeloDocumentoContrato({ row }: IEventParam<Button>) {
        DialogService.show({
            name: 'dialog-deletar-anexo',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente sobrescrever o documento em anexo para o mesmo documento do Modelo de Documento e Idioma informados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.atualizarAnexoModeloDocumentoContrato(row);
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public async atualizarAnexoModeloDocumentoContrato(row : any) {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-contrato');
        LoadingService.show();
        Http.post('/atualizarAnexoModeloDocumentoContrato', {
            row,
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            if (error.response.data.error === 'SEM_MODELO') {
                Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Não foi encontrado um anexo de acordo com o Modelo de Documento e Idioma informado.');
            } else {
                Utilities.showErrorMessage(error);
            }
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
